#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
@Time    : 2023/5/2 17:46
@Author  : alexanderwu
@File    : test_summarize.py
"""

import pytest

CASES = [
    """# 上下文
[{'title': '抗痘 / 控油 / 毛孔調理 臉部保養 商品 | 屈臣氏 Watsons', 'href': 'https://www.watsons.com.tw/%E8%87%89%E9%83%A8%E4%BF%9D%E9%A4%8A/%E6%8A%97%E7%97%98-%E6%8E%A7%E6%B2%B9-%E6%AF%9B%E5%AD%94%E8%AA%BF%E7%90%86/c/10410601', 'body': '抗痘 / 控油 / 毛孔調理等臉部保養用品盡在屈臣氏，多樣抗痘 / 控油 / 毛孔調理商品全面符合您的需求。3M, 3M Nexcare, ARIN, Biore 蜜妮, CEZANNE等眾多推薦品牌快來屈臣氏選購。'}, {'title': '有哪些祛痘印产品曾惊艳过你？ - 知乎', 'href': 'https://www.zhihu.com/question/380098171', 'body': '有哪些祛痘印产品曾惊艳过你？ ... 素姬水杨酸精华 祛痘产品里绝对不能少了水杨酸这个成分!用这个品牌主要是信赖它的温和性，而且价格便宜，去粉刺痘痘效果又好，对闭口和黑头都有效果。 ... 购买比较方便，我在屈臣氏买的，50RMB. 西班牙IFC duo祛痘凝露 ...'}, {'title': '屈臣氏祛痘系列_百度知道', 'href': 'https://zhidao.baidu.com/question/581355167.html', 'body': '2014-08-28 屈臣氏里有哪些祛痘效果好的产品？ 26 2007-08-25 屈臣氏有卖哪些祛痘产品 61 2019-05-27 屈臣氏有哪些祛痘产品 什么方法会比较好？？ 2015-09-27 屈臣氏白金祛痘系列的使用顺序 30 2014-11-03 屈臣氏卖的祛痘产品叫什么 1 2011-05-24 屈臣氏的祛痘好用的产品有那些 ...'}, {'title': '屈臣氏里有哪些祛痘效果好的产品？ - 百度知道', 'href': 'https://zhidao.baidu.com/question/360679400530686652.html', 'body': '阿达帕林是一款医药系列的祛痘产品，它里面蕴含了非常丰富的甲酸类化合物，涂抹在皮肤上会有很好的消炎效果，对于粉刺、闭口、痘痘等痤疮系列的皮肤问题也有很好的修复，可以让毛囊上的皮肤细胞正常分化。. 用户实测评分：9.663分. 实验室效果评测：9. ...'}, {'title': '33款屈臣氏最值得买的好物! - 知乎 - 知乎专栏', 'href': 'https://zhuanlan.zhihu.com/p/31366278', 'body': '屈臣氏深层卸妆棉. 19.9元/25*2. 一般出差不想带很多瓶瓶罐罐就会带卸妆棉，当时是买一送一，就觉得超划算。. 棉质很好，很舒服，厚度适中，温和不刺激，淡淡的香味，卸得很舒心，卸得也很干净。. 眼妆也可以用这个卸，因为它不含酒精，所以一点也不辣 ...'}, {'title': '屈臣氏官网 - Watsons', 'href': 'https://www.watsons.com.cn/', 'body': '屈臣氏百年正品口碑，现金优惠多多多，2小时闪电送到家，还能屈臣氏门店自提。美妆洗护，口腔保健，日用百货，男士护理，更便捷的操作，满足你更多。屈臣氏始创于1841年，线下门店覆盖全球12个国家地区，超过5500家门店。在中国，400多个城市已超过3000家门店，6000万名会员与你一起放心买好货!'}, {'title': '15款日本最具口碑的祛痘神器! - 知乎 - 知乎专栏', 'href': 'https://zhuanlan.zhihu.com/p/63349036', 'body': '乐敦. Acnes药用祛痘抗痘粉尘暗疮药膏. 药用抗痘药膏清爽啫哩質地，维生素E衍生物，维生素B6组合，膏体不腻，轻透很好吸收，淡淡清香味主要针对红肿且疼痛的大颗痘痘，排出脓液、杀灭细菌、消除红肿，第二天就会有效果。. DHC. 祛痘净痘调理精华. 含有o-Cymen ...'}, {'title': '请问屈臣氏什么产品可以去痘疤的 - Sina', 'href': 'https://iask.sina.com.cn/b/1STygN4RT2wZ.html', 'body': '请问屈臣氏什么产品可以去痘疤的本人很少长痘痘，偶尔冒几颗。脸颊上的痘痘来的快去的快，不怎么留疤，就是额头和下巴嘴角边的痘痘感觉超级敏感，一挤就留疤，苦恼! ... 想问下屈臣氏有什么产品能去痘疤的，要有效哦~谢谢各位了! ...'}, {'title': '屈臣氏祛痘凝胶新款 - 屈臣氏祛痘凝胶2021年新款 - 京东', 'href': 'https://www.jd.com/xinkuan/16729c68245569aae4c3.html', 'body': '屈臣氏芦荟凝胶清凉滋润舒缓祛痘印痘坑痘疤补水保湿晒后修复凝胶 【保湿芦荟凝胶】3瓶900g. 2+ 条评论. 屈臣氏 Leaf Simple简单叶子水杨酸祛痘凝胶去痘印粉刺闭口淡化痘坑研春堂收缩毛孔改善粉刺 两支. 4+ 条评论. 屈臣氏 Leaf Simple简单叶子水杨酸祛痘凝胶去痘印 ...'}]

# 用户搜索请求
屈臣氏有什么产品可以去痘？

# 要求
你是专业管家团队的一员，会给出有帮助的建议
1. 请根据上下文，对用户搜索请求进行总结性回答，不要包括与请求无关的文本
2. 以 [正文](引用链接) markdown形式在正文中**自然标注**~5个文本（如商品词或类似文本段），以便跳转
3. 回复优雅、清晰，**绝不重复文本**，行文流畅，长度居中""",
    """# 上下文
[{'title': '去厦门 有哪些推荐的美食？ - 知乎', 'href': 'https://www.zhihu.com/question/286901854', 'body': '知乎，中文互联网高质量的问答社区和创作者聚集的原创内容平台，于 2011 年 1 月正式上线，以「让人们更好的分享知识、经验和见解，找到自己的解答」为品牌使命。知乎凭借认真、专业、友善的社区氛围、独特的产品机制以及结构化和易获得的优质内容，聚集了中文互联网科技、商业、影视 ...'}, {'title': '厦门到底有哪些真正值得吃的美食？ - 知乎', 'href': 'https://www.zhihu.com/question/38012322', 'body': '有几个特色菜在别处不太能吃到，值得一试~常点的有西多士、沙茶肉串、咕老肉（个人认为还是良山排档的更炉火纯青~），因为爱吃芋泥，每次还会点一个芋泥鸭~人均50元左右. 潮福城. 厦门这两年经营港式茶点的店越来越多，但是最经典的还是潮福城的茶点 ...'}, {'title': '超全厦门美食攻略，好吃不贵不踩雷 - 知乎 - 知乎专栏', 'href': 'https://zhuanlan.zhihu.com/p/347055615', 'body': '厦门老字号店铺，味道卫生都有保障，喜欢吃芒果的，不要错过芒果牛奶绵绵冰. 285蚝味馆 70/人. 上过《舌尖上的中国》味道不用多说，想吃地道的海鲜烧烤就来这里. 堂宴.老厦门私房菜 80/人. 非常多的明星打卡过，上过《十二道锋味》，吃厦门传统菜的好去处 ...'}, {'title': '福建名小吃||寻味厦门，十大特色名小吃，你都吃过哪几样？ - 知乎', 'href': 'https://zhuanlan.zhihu.com/p/375781836', 'body': '第一期，分享厦门的特色美食。 厦门是一个风景旅游城市，许多人来到厦门，除了游览厦门独特的风景之外，最难忘的应该是厦门的特色小吃。厦门小吃多种多样，有到厦门必吃的沙茶面、米线糊、蚵仔煎、土笋冻等非常之多。那么，厦门的名小吃有哪些呢？'}, {'title': '大家如果去厦门旅游的话，好吃的有很多，但... 来自庄时利和 - 微博', 'href': 'https://weibo.com/1728715190/MEAwzscRT', 'body': '大家如果去厦门旅游的话，好吃的有很多，但如果只选一样的话，我个人会选择莲花煎蟹。 靠海吃海，吃蟹对于闽南人来说是很平常的一件事。 厦门传统的做法多是清蒸或水煮，上世纪八十年代有一同安人在厦门的莲花公园旁，摆摊做起了煎蟹的生意。'}, {'title': '厦门美食,厦门美食攻略,厦门旅游美食攻略 - 马蜂窝', 'href': 'https://www.mafengwo.cn/cy/10132/gonglve.html', 'body': '醉壹号海鲜大排档 (厦门美食地标店) No.3. 哆啦Eanny 的最新点评：. 环境 挺复古的闽南风情，花砖地板，一楼有海鲜自己点菜，二楼室内位置，三楼露天位置，环境挺不错的。. 苦螺汤，看起来挺清的，螺肉吃起来很脆。. 姜... 5.0 分. 482 条用户点评.'}, {'title': '厦门超强中山路小吃合集，29家本地人推荐的正宗美食 - 马蜂窝', 'href': 'https://www.mafengwo.cn/gonglve/ziyouxing/176485.html', 'body': '莲欢海蛎煎. 提到厦门就想到海蛎煎，而这家位于中山路局口街的莲欢海蛎煎是实打实的好吃!. ·局口街老巷之中，全室外环境，吃的就是这种感觉。. ·取名"莲欢"，是希望妻子每天开心。. 新鲜的食材，实在的用料，这样的用心也定能讨食客欢心。. ·海蛎又 ...'}, {'title': '厦门市 10 大餐厅- Tripadvisor', 'href': 'https://cn.tripadvisor.com/Restaurants-g297407-Xiamen_Fujian.html', 'body': '厦门市餐厅：在Tripadvisor查看中国厦门市餐厅的点评，并以价格、地点及更多选项进行搜索。 ... "牛排太好吃了啊啊啊" ... "厦门地区最老品牌最有口碑的潮州菜餐厅" ...'}, {'title': '#福建10条美食街简直不要太好吃#每到一... 来自新浪厦门 - 微博', 'href': 'https://weibo.com/1740522895/MF1lY7W4n', 'body': '福建的这10条美食街，你一定不能错过!福州师大学生街、福州达明路美食街、厦门八市、漳州古城老街、宁德老南门电影院美食集市、龙岩中山路美食街、三明龙岗夜市、莆田金鼎夜市、莆田玉湖夜市、南平嘉禾美食街。世间万事皆难，唯有美食可以治愈一切。'}, {'title': '厦门这50家餐厅最值得吃 - 腾讯新闻', 'href': 'https://new.qq.com/rain/a/20200114A09HJT00', 'body': '没有什么事是一顿辣解决不了的! 创意辣、川湘辣、温柔辣、异域辣，芙蓉涧的菜能把辣椒玩出花来! ... 早在2005年，这家老牌的东南亚餐厅就开在厦门莲花了，在许多老厦门的心中，都觉得这里有全厦门最好吃的咖喱呢。 ...'}, {'title': '好听的美食？又好听又好吃的食物有什么？ - 哔哩哔哩', 'href': 'https://www.bilibili.com/read/cv23430069/', 'body': '专栏 / 好听的美食？又好听又好吃的食物有什么？ 又好听又好吃的食物有什么？ 2023-05-02 18:01 --阅读 · --喜欢 · --评论'}]

# 用户搜索请求
厦门有什么好吃的？

# 要求
你是专业管家团队的一员，会给出有帮助的建议
1. 请根据上下文，对用户搜索请求进行总结性回答，不要包括与请求无关的文本
2. 以 [正文](引用链接) markdown形式在正文中**自然标注**3-5个文本（如商品词或类似文本段），以便跳转
3. 回复优雅、清晰，**绝不重复文本**，行文流畅，长度居中""",
]


@pytest.mark.usefixtures("llm_api")
def test_summarize(llm_api):
    pass
